#include <errno.h>

#include <csparse.h>
#include <cstats.h>
#include <string.h>
#include "tbx_timer.h"

#include "interface.h"


#ifndef PARALLEL_H
#define PARALLEL_H

#define INFO_STRLEN    20
#define COMMAND_STRLEN 80
#define PGM_STRLEN     80
#define NODE_PGM_NAME  "etmspv3a"

#define INTRO_LETTER   1
#define MATRIX_LETTER  2
#define TIMING_LETTER  3

struct matrix_host_bin
{
   int               M;
   int               N;

   int               P;
   int               Q;

   void            (*Row_Distribution)();
   void            (*Col_Distribution)();
   void             *Row_Extra;
   void             *Col_Extra;

   int             **A_element_Count;
   struct element ***A;
   struct element   *Pool;
};

typedef struct matrix_host_bin matrix_host_bin;

struct element
{
   index  i;
   index  j;
   double Value;

   struct element *Next;
};

typedef struct element element;

struct matrix_cube_bin
{
   int     M;
   int     N;

   int     Array_len;
   int     num_element;
   long   *i;
   long   *j;
   double *Value;
};

typedef struct matrix_cube_bin matrix_cube_bin;

#endif


void parallel_cube_host_(complex *BRAU, complex *BRAL,
                         complex *STAIRS, short *FENO, int *FELI,
                         int *NELI, short *IBUSO, short *PNODIA, int *NBUS);

void read_distribution(FILE *InputFile, short *Dis_Type,
                       char *Extra_param1_name, long *Extra_param1,
                       char *Extra_param2_name, long *Extra_param2);

void set_distribution(ZIP_MAILER *Mailer, short Dis_Type,
                      int rctype, Cdistrib **Dis, void ((**Distribution)()),
                      int ((**Inverse_Distribution)()),
                      int ((**Distribution_limit)()),
                      int ((**Distribution_init)()), int Max_Grid_dim,
                      index Grid_coor, int Global_Mat_dim, int *Local_Mat_dim,
                      int Extra_param1, int Extra_param2, void **Extra);

void Save_matrix_data_diagonal(int I, complex Diag_Value,
                               matrix_host_bin *Matrix);
void Save_matrix_data_off_diagonal(int I, int J, complex Upper_Value,
                                   complex Lower_Value,
                                   matrix_host_bin *Matrix);

element *get_new_element(element **Pool, index i, index j, double Value);

void clear_matrix(matrix_host_bin *Matrix);

void transmit_intro_letter(ZIP_MAILER *Mailer,
                           long Row_Dis_Type, long Col_Dis_Type,
                           long Row_Extra_param1, long Row_Extra_param2,
                           long Col_Extra_param1, long Col_Extra_param2,
                           long Strategy);

void receive_intro_letter(ZIP_MAILER *Mailer,
                          short *Row_Dis_Type, short *Col_Dis_Type,
                          long *Row_Extra_param1, long *Row_Extra_param2,
                          long *Col_Extra_param1, long *Col_Extra_param2,
                          long *Strategy);

void transmit_matrix_letter(ZIP_MAILER *Mailer, matrix_host_bin *Matrix);
void receive_matrix_letter(ZIP_MAILER *Mailer, matrix_cube_bin *Matrix);

void transmit_timing_letter(ZIP_MAILER *Mailer, tbx_timer *Timer);
int timing_max_fn(long *ptr1, long *ptr2, int size, int nitems);
void receive_timing_letter(ZIP_MAILER *Mailer, long *sec, long *usec);

void linear_system(Dlu_info *Info, Cvector *b, Cvector *x, Extra *extra);

